@@ -1,18 +1,28 @@
-import time
+import time, os
+from functools import partial
+import copy
+
+from PyQt5.QtCore import Qt, pyqtSignal
+from PyQt5.QtWidgets import QPushButton, QLabel, QVBoxLayout, QWidget, QGridLayout
+
+from electrum.gui.qt.util import WindowModalDialog, CloseButton, get_parent_main_window, Buttons
+from electrum.gui.qt.transaction_dialog import TxDialog
 
 from electrum.i18n import _
 from electrum.plugin import hook
-from electrum.wallet import Standard_Wallet
-from electrum.gui.qt.util import *
+from electrum.wallet import Multisig_Wallet
+from electrum.transaction import PartialTransaction
 
-from .coldcard import ColdcardPlugin
+from .coldcard import ColdcardPlugin, xfp2str
 from ..hw_wallet.qt import QtHandlerBase, QtPluginBase
 from ..hw_wallet.plugin import only_hook_if_libraries_available
 
 
+CC_DEBUG = False
+
 class Plugin(ColdcardPlugin, QtPluginBase):
-    icon_unpaired = ":icons/coldcard_unpaired.png"
-    icon_paired = ":icons/coldcard.png"
+    icon_unpaired = "coldcard_unpaired.png"
+    icon_paired = "coldcard.png"
 
     def create_handler(self, window):
         return Coldcard_Handler(window)
@@ -20,75 +30,57 @@ class Plugin(ColdcardPlugin, QtPluginBase):
     @only_hook_if_libraries_available
     @hook
     def receive_menu(self, menu, addrs, wallet):
-        if type(wallet) is not Standard_Wallet:
+        # Context menu on each address in the Addresses Tab, right click...
+        if len(addrs) != 1:
             return
-        keystore = wallet.get_keystore()
-        if type(keystore) == self.keystore_class and len(addrs) == 1:
-            def show_address():
-                keystore.thread.add(partial(self.show_address, wallet, addrs[0]))
-            menu.addAction(_("Show on Coldcard"), show_address)
+        for keystore in wallet.get_keystores():
+            if type(keystore) == self.keystore_class:
+                def show_address(keystore=keystore):
+                    keystore.thread.add(partial(self.show_address, wallet, addrs[0], keystore=keystore))
+                device_name = "{} ({})".format(self.device, keystore.label)
+                menu.addAction(_("Show on {}").format(device_name), show_address)
 
     @only_hook_if_libraries_available
     @hook
-    def transaction_dialog(self, dia):
-        # see gui/qt/transaction_dialog.py
+    def wallet_info_buttons(self, main_window, dialog):
+        # user is about to see the "Wallet Information" dialog
+        # - add a button if multisig wallet, and a Coldcard is a cosigner.
+        wallet = main_window.wallet
 
-        keystore = dia.wallet.get_keystore()
-        if type(keystore) != self.keystore_class:
-            # not a Coldcard wallet, hide feature
+        if type(wallet) is not Multisig_Wallet:
             return
 
-        # - add a new button, near "export"
-        btn = QPushButton(_("Save PSBT"))
-        btn.clicked.connect(lambda unused: self.export_psbt(dia))
-        if dia.tx.is_complete():
-            # but disable it for signed transactions (nothing to do if already signed)
-            btn.setDisabled(True)
-
-        dia.sharing_buttons.append(btn)
-
-    def export_psbt(self, dia):
-        # Called from hook in transaction dialog
-        tx = dia.tx
-
-        if tx.is_complete():
-            # if they sign while dialog is open, it can transition from unsigned to signed,
-            # which we don't support here, so do nothing
+        if not any(type(ks) == self.keystore_class for ks in wallet.get_keystores()):
+            # doesn't involve a Coldcard wallet, hide feature
             return
 
-        # can only expect Coldcard wallets to work with these files (right now)
-        keystore = dia.wallet.get_keystore()
-        assert type(keystore) == self.keystore_class
+        btn = QPushButton(_("Export for Coldcard"))
+        btn.clicked.connect(lambda unused: self.export_multisig_setup(main_window, wallet))
 
-        # convert to PSBT
-        raw_psbt = keystore.build_psbt(tx, wallet=dia.wallet)
+        return btn
 
-        name = (dia.wallet.basename() + time.strftime('-%y%m%d-%H%M.psbt')).replace(' ', '-')
-        fileName = dia.main_window.getSaveFileName(_("Select where to save the PSBT file"),
-                                                        name, "*.psbt")
+    def export_multisig_setup(self, main_window, wallet):
+
+        basename = wallet.basename().rsplit('.', 1)[0]        # trim .json
+        name = f'{basename}-cc-export.txt'.replace(' ', '-')
+        fileName = main_window.getSaveFileName(_("Select where to save the setup file"),
+                                                        name, "*.txt")
         if fileName:
-            with open(fileName, "wb+") as f:
-                f.write(raw_psbt)
-            dia.show_message(_("Transaction exported successfully"))
-            dia.saved = True
+            with open(fileName, "wt") as f:
+                ColdcardPlugin.export_ms_wallet(wallet, f, basename)
+            main_window.show_message(_("Wallet setup file exported successfully"))
 
     def show_settings_dialog(self, window, keystore):
         # When they click on the icon for CC we come here.
-        device_id = self.choose_device(window, keystore)
-        if device_id:
-            CKCCSettingsDialog(window, self, keystore, device_id).exec_()
+        # - doesn't matter if device not connected, continue
+        CKCCSettingsDialog(window, self, keystore).exec_()
 
 
 class Coldcard_Handler(QtHandlerBase):
-    setup_signal = pyqtSignal()
-    #auth_signal = pyqtSignal(object)
 
     def __init__(self, win):
         super(Coldcard_Handler, self).__init__(win, 'Coldcard')
-        self.setup_signal.connect(self.setup_dialog)
-        #self.auth_signal.connect(self.auth_dialog)
 
-    
     def message_dialog(self, msg):
         self.clear_dialog()
         self.dialog = dialog = WindowModalDialog(self.top_level_window(), _("Coldcard Status"))
@@ -96,33 +88,28 @@ class Coldcard_Handler(QtHandlerBase):
         vbox = QVBoxLayout(dialog)
         vbox.addWidget(l)
         dialog.show()
-        
-    def get_setup(self):
-        self.done.clear()
-        self.setup_signal.emit()
-        self.done.wait()
-        return 
-        
-    def setup_dialog(self):
-        self.show_error(_('Please initialize your Coldcard while disconnected.'))
-        return
+
 
 class CKCCSettingsDialog(WindowModalDialog):
-    '''This dialog doesn't require a device be paired with a wallet.
-    We want users to be able to wipe a device even if they've forgotten
-    their PIN.'''
 
-    def __init__(self, window, plugin, keystore, device_id):
+    def __init__(self, window, plugin, keystore):
         title = _("{} Settings").format(plugin.device)
         super(CKCCSettingsDialog, self).__init__(window, title)
         self.setMaximumWidth(540)
 
+        # Note: Coldcard may **not** be connected at present time. Keep working!
+
         devmgr = plugin.device_manager()
-        config = devmgr.config
-        handler = keystore.handler
+        #config = devmgr.config
+        #handler = keystore.handler
         self.thread = thread = keystore.thread
+        self.keystore = keystore
 
         def connect_and_doit():
+            # Attempt connection to device, or raise.
+            device_id = plugin.choose_device(window, keystore)
+            if not device_id:
+                raise RuntimeError("Device not connected")
             client = devmgr.client_by_id(device_id)
             if not client:
                 raise RuntimeError("Device not connected")
@@ -144,13 +131,14 @@ class CKCCSettingsDialog(WindowModalDialog):
         y = 3
 
         rows = [
+            ('xfp', _("Master Fingerprint")),
+            ('serial', _("USB Serial")),
             ('fw_version', _("Firmware Version")),
             ('fw_built', _("Build Date")),
             ('bl_version', _("Bootloader")),
-            ('xfp', _("Master Fingerprint")),
-            ('serial', _("USB Serial")),
         ]
         for row_num, (member_name, label) in enumerate(rows):
+            # XXX we know xfp already, even if not connected
             widget = QLabel('<tt>000000000000')
             widget.setTextInteractionFlags(Qt.TextSelectableByMouse | Qt.TextSelectableByKeyboard)
 
@@ -160,7 +148,7 @@ class CKCCSettingsDialog(WindowModalDialog):
             y += 1
         body_layout.addLayout(grid)
 
-        upg_btn = QPushButton('Upgrade')
+        upg_btn = QPushButton(_('Upgrade'))
         #upg_btn.setDefault(False)
         def _start_upgrade():
             thread.add(connect_and_doit, on_success=self.start_upgrade)
@@ -173,13 +161,22 @@ class CKCCSettingsDialog(WindowModalDialog):
         dialog_vbox = QVBoxLayout(self)
         dialog_vbox.addWidget(body)
 
-        # Fetch values and show them
-        thread.add(connect_and_doit, on_success=self.show_values)
+        # Fetch firmware/versions values and show them.
+        thread.add(connect_and_doit, on_success=self.show_values, on_error=self.show_placeholders)
+
+    def show_placeholders(self, unclear_arg):
+        # device missing, so hide lots of detail.
+        self.xfp.setText('<tt>%s' % self.keystore.get_root_fingerprint())
+        self.serial.setText('(not connected)')
+        self.fw_version.setText('')
+        self.fw_built.setText('')
+        self.bl_version.setText('')
 
     def show_values(self, client):
+
         dev = client.dev
 
-        self.xfp.setText('<tt>0x%08x' % dev.master_fingerprint)
+        self.xfp.setText('<tt>%s' % xfp2str(dev.master_fingerprint))
         self.serial.setText('<tt>%s' % dev.serial)
 
         # ask device for versions: allow extras for future
