@@ -8,7 +8,7 @@ from electrum.plugin import hook
 from electrum.wallet import Standard_Wallet
 from electrum.gui.qt.util import WindowModalDialog
 
-from .ledger import LedgerPlugin
+from .ledger import LedgerPlugin, Ledger_Client
 from ..hw_wallet.qt import QtHandlerBase, QtPluginBase
 from ..hw_wallet.plugin import only_hook_if_libraries_available
 
@@ -33,7 +33,7 @@ class Plugin(LedgerPlugin, QtPluginBase):
 
 class Ledger_Handler(QtHandlerBase):
     setup_signal = pyqtSignal()
-    auth_signal = pyqtSignal(object)
+    auth_signal = pyqtSignal(object, object)
 
     def __init__(self, win):
         super(Ledger_Handler, self).__init__(win, 'Ledger')
@@ -56,20 +56,20 @@ class Ledger_Handler(QtHandlerBase):
         vbox.addWidget(l)
         dialog.show()
 
-    def auth_dialog(self, data):
+    def auth_dialog(self, data, client: 'Ledger_Client'):
         try:
             from .auth2fa import LedgerAuthDialog
         except ImportError as e:
             self.message_dialog(repr(e))
             return
-        dialog = LedgerAuthDialog(self, data)
+        dialog = LedgerAuthDialog(self, data, client=client)
         dialog.exec_()
         self.word = dialog.pin
         self.done.set()
 
-    def get_auth(self, data):
+    def get_auth(self, data, *, client: 'Ledger_Client'):
         self.done.clear()
-        self.auth_signal.emit(data)
+        self.auth_signal.emit(data, client)
         self.done.wait()
         return self.word
 
