@@ -794,12 +794,12 @@ class WalletDB(JsonDB):
         return list(d.items())
 
     @locked
-    def get_txo_addr(self, tx_hash: str, address: str) -> Iterable[Tuple[int, int, bool]]:
-        """Returns an iterable of (output_index, value, is_coinbase)."""
+    def get_txo_addr(self, tx_hash: str, address: str) -> Dict[int, Tuple[int, bool]]:
+        """Returns a dict: output_index -> (value, is_coinbase)."""
         assert isinstance(tx_hash, str)
         assert isinstance(address, str)
         d = self.txo.get(tx_hash, {}).get(address, {})
-        return [(int(n), v, cb) for (n, (v, cb)) in d.items()]
+        return {int(n): (v, cb) for (n, (v, cb)) in d.items()}
 
     @modifier
     def add_txi_addr(self, tx_hash: str, addr: str, ser: str, v: int) -> None:
@@ -1051,7 +1051,7 @@ class WalletDB(JsonDB):
         self.tx_fees.pop(txid, None)
 
     @locked
-    def get_dict(self, name):
+    def get_dict(self, name) -> dict:
         # Warning: interacts un-intuitively with 'put': certain parts
         # of 'data' will have pointers saved as separate variables.
         if name not in self.data:
@@ -1211,6 +1211,14 @@ class WalletDB(JsonDB):
             v = Outpoint(**v)
         return v
 
+    def _should_convert_to_stored_dict(self, key) -> bool:
+        if key == 'keystore':
+            return False
+        multisig_keystore_names = [('x%d/' % i) for i in range(1, 16)]
+        if key in multisig_keystore_names:
+            return False
+        return True
+
     def write(self, storage: 'WalletStorage'):
         with self.lock:
             self._write(storage)
